package controllers

import (
	"fmt"
	"github.com/kataras/iris"
	"lapp_-wy/utils"
	"lapp_-wy/web/middleware/glog"
	"lapp_-wy/web/middleware/jwts"
	"lapp_-wy/web/models"
	"lapp_-wy/web/supports"
	"time"
)

func GetChargetabList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargetabList"
	logs.Message = "收费记录列表查看"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	var data models.Chargetab
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Cid = user.Pid
	data.Chargetype, _ = ctx.URLParamInt("chargetype")
	buildingid := ctx.URLParam("buildingid")
	unit := ctx.URLParam("unit")
	room := ctx.URLParam("room")
	data.Chargestartdate = ctx.URLParam("startdate")
	data.Chargeenddate = ctx.URLParam("enddate")
	data.Propertyid = ctx.URLParam("propertyid")
	result, count, err := data.GetPage(pageSize, pageIndex, buildingid, unit, room)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["user"] = user.Role
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetChargetab(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	me.Chargenr, _ = ctx.URLParamInt("chargenr")
	result, err := me.SelectOne()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

//按照年统计
func GetChargetabByYear(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	me.Chargetype, _ = ctx.URLParamInt("chargetype")
	beginyear := ctx.URLParam("startYear")
	endyear := ctx.URLParam("endYear")
	accountMonth := ctx.URLParam("accountMonth")
	settledate := utils.ValueToInt(accountMonth, 0)
	result, count, err := me.ChargeTotalByYear(beginyear, endyear, settledate)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	supports.Ok(ctx, supports.OptionSuccess, res)
}

//按楼栋统计
func GetChargetabByBuilding(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	me.Chargetype, _ = ctx.URLParamInt("chargetype")
	buildingid := ctx.URLParam("building")
	beginyear := ctx.URLParam("startYear")
	accountMonth := ctx.URLParam("accountMonth")
	settledate := utils.ValueToInt(accountMonth, 0)
	endyear := utils.ValueToInt(beginyear, 0) + 1
	result, count, rate, err := me.ChargeTotalByBuilding(buildingid, beginyear, utils.ValueToString(endyear, ""), settledate)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["rate"] = rate
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetChargetabListPage(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargetabList"
	logs.Message = "收费记录列表查看"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	var data models.Chargetab
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Cid = user.Pid
	chargetype, _ := ctx.URLParamInt("chargetype")
	data.Chargetype = chargetype
	switch chargetype {
	case 1:
		data.Propertyid = ctx.URLParam("number")
	case 2:
		data.Accesscardid = ctx.URLParam("number")
	case 3:
		data.Carportid = ctx.URLParam("number")
	}
	searchtime := ctx.URLParam("expireDate")
	startDate := ctx.URLParam("startDate")
	result, count, total, err := data.ListPage(pageSize, pageIndex, startDate, searchtime)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["total"] = total
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetChargetabReadExcel(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargetabList"
	logs.Message = "收费记录列表查看"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	var data models.Chargetab
	var err error
	data.Cid = user.Pid
	chargetype, _ := ctx.URLParamInt("chargetype")
	data.Chargetype = chargetype
	searchtime := ctx.URLParam("expireDate")
	startDate := ctx.URLParam("startDate")
	_, filename, err := data.ReadExcel(startDate, searchtime)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}

func GetChargetabReadBuildingExcel(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var data models.Chargetab
	var err error
	data.Cid = user.Pid
	buildingid := ctx.URLParam("building")
	beginyear := ctx.URLParam("startYear")
	accountMonth := ctx.URLParam("accountMonth")
	settledate := utils.ValueToInt(accountMonth, 0)
	endyear := utils.ValueToInt(beginyear, 0) + 1
	_, filename, err := data.ReadBuildExcel(buildingid, beginyear, utils.ValueToString(endyear, ""), settledate)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}

func GetChargetabReadYearExcel(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	me.Chargetype, _ = ctx.URLParamInt("chargetype")
	beginyear := ctx.URLParam("startYear")
	endyear := ctx.URLParam("endYear")
	accountMonth := ctx.URLParam("accountMonth")
	settledate := utils.ValueToInt(accountMonth, 0)
	_, filename, err := me.ReadYearExcel(beginyear, endyear, settledate)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}

func GetChargetabReadDataExcel(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	me.Chargetype, _ = ctx.URLParamInt("chargetype")
	buildingid := ctx.URLParam("buildingid")
	unit := ctx.URLParam("unit")
	room := ctx.URLParam("room")
	me.Chargestartdate = ctx.URLParam("startdate")
	me.Chargeenddate = ctx.URLParam("enddate")
	_, filename, err := me.ReadChargeDataExcel(buildingid, unit, room)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}

func DelChargetab(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "DelChargetab"
	logs.Message = "删除缴费记录"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	data := new(models.Chargetab)
	data.Cid = user.Pid
	data.Chargenr, _ = ctx.URLParamInt("chargenr")
	data.Lastmodifyby = user.Userid
	data.Serialnumber = ctx.URLParam("serialnumber")
	err := data.Del()
	if err != nil {
		glog.InfoExt("删除缴费记录", "错误原因:", err)
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}

	supports.Ok(ctx, "删除成功", "")
}

func GetRecordLogList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetRecordLogList"
	logs.Message = "删除记录列表查看"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	var data models.RecordLog
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Cid = user.Pid
	data.Createby = ctx.URLParam("createby")
	startdate := ctx.URLParam("startdate")
	enddate := ctx.URLParam("enddate")

	result, count, err := data.GetPage(pageSize, pageIndex, startdate, enddate)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

// 获取缴费台账统计数据
func GetChargeStatisticData(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargeStatisticData"
	logs.Message = "当年及陈欠缴费统计查看"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()

	var cid int
	// 如果公司机关部门可以搜索cid
	if user.Pid == 110 {
		cidStr := ctx.URLParam("cid")
		cid = utils.ValueToInt(cidStr, 0)
	} else {
		cid = user.Pid
	}
	if cid == 0 {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	chargeType, err := ctx.URLParamInt("chargetype")
	if err != nil {
		chargeType = 0
	}
	buildingId := ctx.URLParam("buildingid")
	propertyId := ctx.URLParam("propertyid")

	start := ctx.URLParam("start")
	end := ctx.URLParam("end")
	if start != "" || end != "" {
		startT, err1 := utils.TimeParseyyyyMMdd(start)
		endT, err2 := utils.TimeParseyyyyMMdd(end)
		if err1 != nil || err2 != nil {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}
		start = utils.TimeFormat(startT, "yyyy-MM-dd")
		end = utils.TimeFormat(endT, "yyyy-MM-dd")
	}
	currentYear := time.Now().Year()
	var year int
	yearStr := ctx.URLParam("year")
	if yearStr != "" {
		year = utils.ValueToInt(yearStr, 0)
	} else {
		year = 0
	}
	selectType := ctx.URLParam("type")
	if selectType != "arrearage" && selectType != "current" {
		selectType = "total"
	}
	var charge models.Chargetab
	result, err := charge.GetChargeStatisticData(cid, year, selectType, chargeType, buildingId, propertyId, start, end, currentYear)
	if err != nil {
		supports.Error(ctx, iris.StatusInternalServerError, "抱歉未找到相关信息", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

// 获取缴费台账
func GetChargeArrearageData(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargeArrearageData"
	logs.Message = "按日期查看缴费陈欠数据"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()

	var pageSize = 10
	var pageIndex = 1
	if size := ctx.URLParam("size"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("index"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}

	var cid int
	// 如果公司机关部门可以搜索cid
	if user.Pid == 110 {
		cidStr := ctx.URLParam("cid")
		cid = utils.ValueToInt(cidStr, 0)
	} else {
		cid = user.Pid
	}
	if cid == 0 {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	chargeType, err := ctx.URLParamInt("chargetype")
	if err != nil {
		chargeType = 0
	}
	buildingId := ctx.URLParam("buildingid")
	propertyId := ctx.URLParam("propertyid")

	start := ctx.URLParam("start")
	end := ctx.URLParam("end")
	if start != "" || end != "" {
		startT, err1 := utils.TimeParseyyyyMMdd(start)
		endT, err2 := utils.TimeParseyyyyMMdd(end)
		if err1 != nil || err2 != nil {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}
		start = utils.TimeFormat(startT, "yyyy-MM-dd")
		end = utils.TimeFormat(endT, "yyyy-MM-dd")
	}
	currentYear := time.Now().Year()
	var year int
	yearStr := ctx.URLParam("year")
	if yearStr != "" {
		year = utils.ValueToInt(yearStr, 0)
	} else {
		year = 0
	}
	arrearageDate := fmt.Sprintf("%d-12-31", currentYear-1)
	var charge models.Chargetab
	result, err := charge.GetChargeArrearageData(cid, pageIndex, pageSize, year, arrearageDate, chargeType, buildingId, propertyId, start, end, currentYear)
	if err != nil {
		fmt.Println("outer error:", err)
		supports.Error(ctx, iris.StatusInternalServerError, "抱歉未找到相关信息", nil)
		return
	}
	result.User = user.Role
	supports.Ok(ctx, supports.OptionSuccess, result)
}

// 按日期和缴费类型获取缴费数据
func GetChargeComputeMoney(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}

	beginTime := ctx.URLParam("beginDate")
	unitPrice, err := ctx.URLParamFloat64("unitPrice")
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "unitPrice获取失败！", nil)
		return
	}
	months, err := ctx.URLParamInt("months")
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "months获取失败！", nil)
		return
	}
	lenDay, err := ctx.URLParamInt("lenDay")
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "lenDay获取失败！", nil)
		return
	}
	area, err := ctx.URLParamFloat64("area")
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "area获取失败！", nil)
		return
	}
	beginDate, err := utils.TimeParseyyyyMMdd(beginTime)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "beginDate时间转换失败！", nil)
		return
	}
	result := utils.ComputeMoney(beginDate, unitPrice, months, lenDay, area)
	supports.Ok(ctx, supports.OptionSuccess, result)
}

// 按日期和缴费类型获取缴费数据
func GetChargeWayListData(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargeWayListData"
	logs.Message = "按日期和收款类型查看缴费数据"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()

	var pageSize = 10
	var pageIndex = 1
	if size := ctx.URLParam("size"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("index"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}

	var cid int
	// 如果公司机关部门可以搜索cid
	if user.Pid == 110 {
		cidStr := ctx.URLParam("cid")
		cid = utils.ValueToInt(cidStr, 0)
	} else {
		cid = user.Pid
	}
	if cid == 0 {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	start := ctx.URLParam("start")
	end := ctx.URLParam("end")
	if start != "" || end != "" {
		startT, err1 := utils.TimeParseyyyyMMdd(start)
		endT, err2 := utils.TimeParseyyyyMMdd(end)
		if err1 != nil || err2 != nil {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}
		start = utils.TimeFormat(startT, "yyyy-MM-dd")
		end = utils.TimeFormat(endT, "yyyy-MM-dd")
	}
	chargeWay := ctx.URLParam("chargeWay")
	chargeType := ctx.URLParam("chargeType")
	var charge models.Chargetab
	result, err := charge.GetChargeWayDataList(cid, chargeWay, chargeType, start, end, pageIndex, pageSize)
	if err != nil {
		supports.Error(ctx, iris.StatusInternalServerError, "抱歉未找到相关信息", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)

}

func ExportChargeArrearageExcel(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargeArrearageData"
	logs.Message = "按日期查看缴费陈欠数据"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()

	var cid int
	// 如果公司机关部门可以搜索cid
	if user.Pid == 110 {
		cidStr := ctx.URLParam("cid")
		cid = utils.ValueToInt(cidStr, 0)
	} else {
		cid = user.Pid
	}
	if cid == 0 {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	chargeType, err := ctx.URLParamInt("chargetype")
	if err != nil {
		chargeType = 0
	}
	buildingId := ctx.URLParam("buildingid")
	propertyId := ctx.URLParam("propertyid")

	start := ctx.URLParam("start")
	end := ctx.URLParam("end")
	if start != "" || end != "" {
		startT, err1 := utils.TimeParseyyyyMMdd(start)
		endT, err2 := utils.TimeParseyyyyMMdd(end)
		if err1 != nil || err2 != nil {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}
		start = utils.TimeFormat(startT, "yyyy-MM-dd")
		end = utils.TimeFormat(endT, "yyyy-MM-dd")
	}
	currentYear := time.Now().Year()
	var year int
	yearStr := ctx.URLParam("year")
	if yearStr != "" {
		year = utils.ValueToInt(yearStr, 0)
	} else {
		year = 0
	}
	arrearageDate := fmt.Sprintf("%d-12-31", currentYear-1)
	var charge models.Chargetab
	result, err := charge.GetChargeArrearageExcelData(cid, year, arrearageDate, chargeType, buildingId, propertyId, start, end, currentYear)
	if err != nil {
		supports.Error(ctx, iris.StatusInternalServerError, "抱歉未找到相关信息", nil)
		return
	}

	_, filename, err := charge.ExportChargeArrearageExcel(result)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}

// 按日期和缴费类型获取缴费数据
func ExportChargeWayDataListExcel(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "ExportChargeWayDataListExcel"
	logs.Message = "导出按日期和收款类型查看缴费数据"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()

	var cid int
	// 如果公司机关部门可以搜索cid
	if user.Pid == 110 {
		cidStr := ctx.URLParam("cid")
		cid = utils.ValueToInt(cidStr, 0)
	} else {
		cid = user.Pid
	}
	if cid == 0 {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	start := ctx.URLParam("start")
	end := ctx.URLParam("end")
	if start != "" || end != "" {
		startT, err1 := utils.TimeParseyyyyMMdd(start)
		endT, err2 := utils.TimeParseyyyyMMdd(end)
		if err1 != nil || err2 != nil {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
			return
		}
		start = utils.TimeFormat(startT, "yyyy-MM-dd")
		end = utils.TimeFormat(endT, "yyyy-MM-dd")
	}
	chargeWay := ctx.URLParam("chargeWay")
	chargeType := ctx.URLParam("chargeType")
	var charge models.Chargetab
	result, err := charge.GetExcelChargeWayDataList(cid, chargeWay, chargeType, start, end)
	if err != nil {
		fmt.Println("error1:", err)
		supports.Error(ctx, iris.StatusInternalServerError, "抱歉未找到相关信息", nil)
		return
	}
	_, filename, err := charge.ExportChargeWayExcel(result)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)

}

func GetChargetabStatisticByYear(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	me.Chargetype, _ = ctx.URLParamInt("chargetype")
	year, err := ctx.URLParamInt("year")
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	selectType := ctx.URLParam("type")
	result, count, err := me.ChargeTotalStatisticData(year, selectType)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	supports.Ok(ctx, supports.OptionSuccess, res)
}

//多园区物业费数据对比
func GetCommunityChargeTotal(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	cids := ctx.URLParam("cids")
	yearDate, _ := ctx.URLParamInt("yearDate")
	result, err := me.CommunityChargeTotal(cids, yearDate)
	if err != nil {
		fmt.Println(111)
		fmt.Println(err)
		fmt.Println(111)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

//多园区报销数据统计
func GetTotalChargeByCommunity(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	cids := ctx.URLParam("cids")
	yearDate := ctx.URLParam("yearDate")
	result, err := me.TotalChargeByCommunity(cids, yearDate)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

//单园区报销数据统计
func GetTotalCatApply(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	cid, _ := ctx.URLParamInt("cid")
	if cid > 0 {
		me.Cid = cid
	} else {
		me.Cid = user.Pid
	}
	lenMonths, _ := ctx.URLParamInt("lenMonths")
	yearDate := ctx.URLParam("yearDate")
	result, err := me.TotalCatApply(yearDate, lenMonths)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

//单园区近年收费流水数据统计
func GetTotalChargeByYear(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	cid, _ := ctx.URLParamInt("cid")
	if cid > 0 {
		me.Cid = cid
	} else {
		me.Cid = user.Pid
	}
	yearDate := ctx.URLParam("yearDate")
	result, err := me.TotalChargeByYear(yearDate)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

//单园区月收费流水数据统计
func GetChargeTotalPage(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}

	var me models.Chargetab
	cid, _ := ctx.URLParamInt("cid")
	if cid > 0 {
		me.Cid = cid
	} else {
		me.Cid = user.Pid
	}
	startDate := ctx.URLParam("startDate")

	ownStartDate := ctx.URLParam("ownStartDate")
	ownEndDate := ctx.URLParam("ownEndDate")
	me.Chargetype, _ = ctx.URLParamInt("chargetype")
	result, count, total, err := me.ChargeTotalPage(pageSize, pageIndex, startDate, ownStartDate, ownEndDate)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}

	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["total"] = total
	supports.Ok(ctx, supports.OptionSuccess, res)
}

//欠费分布图
func GetOweChargeTotal(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	cid, _ := ctx.URLParamInt("cid")
	if cid > 0 {
		me.Cid = cid
	} else {
		me.Cid = user.Pid
	}
	startDate := ctx.URLParam("startDate")
	me.Chargetype, _ = ctx.URLParamInt("chargetype")
	result, err := me.OweChargeTotal(startDate)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}
func GetPropertyFeeData(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	cid, _ := ctx.URLParamInt("cid")
	if cid > 0 {
		me.Cid = cid
	} else {
		me.Cid = user.Pid
	}
	startDate := ctx.URLParam("startDate")
	me.Chargetype = 1
	endDate := ctx.URLParam("endDate")
	result, err := me.PropertyFeeData(startDate, endDate)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}
